import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';

export interface GraphQueryResultFromBackend {
  nodes: {
    id: string;
    attributes: { [key: string]: unknown };
  }[];

  links: {
    attributes: { [key: string]: unknown };
    from: string;
    to: string;
  }[];

  // TODO: Also include type in node and edge
  // TODO: The backend should send all the different entitytypes and relationtypes in the result
}

export interface Node {
  id: string;
  attributes: { [key: string]: unknown };
  /* type: string[]; */
}
export interface Edge {
  attributes: { [key: string]: unknown };
  from: string;
  to: string;
  /* type: string; */
}

// Define a type for the slice state
export interface GraphQueryResult {
  nodes: Node[];
  edges: Edge[];

  // Describes what entities there are in this graph query result.
  nodeTypes: string[];
}

// Define the initial state using that type
const initialState: GraphQueryResult = {
  nodes: [],
  edges: [],
  nodeTypes: [],
};

export const graphQueryResultSlice = createSlice({
  name: 'graphQueryResult',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    assignNewGraphQueryResult: (
      state,
      action: PayloadAction<GraphQueryResultFromBackend>
    ) => {
      // Maybe do some data quality checking and parsing
      // ...

      // Collect all the different nodetypes in the result
      const nodeTypes: string[] = [];
      action.payload.nodes.forEach((node) => {
        // Note: works only for arangodb
        const nodeType = node.id.split('/')[0];
        if (!nodeTypes.includes(nodeType)) nodeTypes.push(nodeType);
      });

      // Assign new state
      state.nodes = action.payload.nodes;
      state.edges = action.payload.links;
      state.nodeTypes = nodeTypes;
    },
  },
});

export const { assignNewGraphQueryResult } = graphQueryResultSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const selectGraphQueryResult = (state: RootState) =>
  state.graphQueryResult;
export const selectGraphQueryResultNodes = (state: RootState) =>
  state.graphQueryResult.nodes;
export const selectGraphQueryResultLinks = (state: RootState) =>
  state.graphQueryResult.edges;

export default graphQueryResultSlice.reducer;
